# -*- coding: utf-8 -*-
from functools import wraps
from luckydonaldUtils.eastereggs.headers import get_headers
from django.utils.decorators import available_attrs

__author__ = 'luckydonald'


def headers(*headers_as_dict, **headers_as_kwargs):
	"""Decorator adding arbitrary HTTP headers to the response.

	This decorator adds HTTP headers specified in the argument (map), to the
	HTTPResponse returned by the function being decorated.

	Example:

	>>> @headers({'Refresh': '10', 'X-Bender': 'Bite my shiny, metal ass!'})
	>>> @headers(Refresh='10', X_Bender='Bite my shiny, metal ass!')
	>>> def index(request):
	>>> 	pass  # your stuff.

	Modified from https://djangosnippets.org/snippets/275/
	"""
	def headers_wrapper(fun):
		@wraps(fun, assigned=available_attrs(fun))
		def wrapped_function(*args, **kwargs):
			response = fun(*args, **kwargs)
			for arg in headers_as_dict:
				if isinstance(arg, dict):
					for k,v in headers_as_kwargs.items():
						response[k] = v
				else:
					raise TypeError("Parameter is not type dict but {}".format(type(arg)))
			for kwarg_key, kwarg_value in headers_as_kwargs.items():
				if not isinstance(kwarg_value, str):
					#try:
					#	temp = str(kwarg_value)
					#	kwarg_value = temp
					#except:
						raise TypeError("Keyword-parameter '{param}' is not type str but {type}, and str() failed.".format(param=kwarg_key, type=type(kwarg_value)))
				if isinstance(kwarg_value, str):
					response[kwarg_key.replace("_","-")] = kwarg_value
				else:
					raise TypeError("Keyword-parameter '{param}' is not type str but {type}.".format(param=kwarg_key, type=type(kwarg_value)))
			return response
		return wrapped_function
	return headers_wrapper

def header(name, value):
	# View decorator that sets a response header.
	#
	# Example:
	# @header('X-Powered-By', 'Django')
	# def view(request, ...):
	#     ....
	#
	# For class-based views use:
	# @method_decorator(header('X-Powered-By', 'Django'))
	# def get(self, request, ...)
	#     ...
	#
	# Taken from https://djangosnippets.org/snippets/2895/
	def decorator(func):
		@wraps(func, assigned=available_attrs(func))
		def inner(request, *args, **kwargs):
			response = func(request, *args, **kwargs)
			response[name] = value
			return response
		return inner
	return decorator


def easteregg_headers(func):
	return headers(get_headers())(func)



# -*- coding: utf-8 -*-
__author__ = 'luckydonald'
from django.middleware.csrf import CsrfViewMiddleware

def check_csrf(request):
	reason = CsrfViewMiddleware().process_view(request, None, (), {})
	if reason:
		# CSRF failed
		return False
	return True



# -*- coding: utf-8 -*-
from datetime import datetime
from . import py2

__author__ = 'luckydonald'


def to_jsonable_dict(skip_keys=list()):
	dict(to_jsonable_dict(skip_keys=skip_keys))

if py2:
	def to_jsonable_dict_iterator(dict, skip_keys=list()):
		for key, value in dict.iteritems():
			if key in skip_keys:
				continue
			if isinstance(value, datetime):
				yield (key, str(value.isoformat()))
			else:
				yield (key, value)
else:
	def to_jsonable_dict_iterator(dict, skip_keys=list()):
		for key, value in dict.items():
			if key in skip_keys:
				continue
			if isinstance(value, datetime):
				yield (key, str(value.isoformat()))
			else:
				yield (key, value)


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
__author__ = 'luckydonald'
import sys

if sys.version < '3': # python 2.7
	text_type = unicode
	binary_type = str
	native_type = binary_type
	unicode_type = text_type  # because I can't remember to use text_type
	long_int = long
	def to_native(x):
		return to_binary(x)
	#end def
else: # python 3
	text_type = str
	binary_type = bytes
	native_type = text_type
	unicode_type = text_type
	long_int = int
	def to_native(x):
		return to_unicode(x)
	#end def
#end if

def to_binary(x):
	if isinstance(x, text_type):
		return x.encode("utf-8")
	elif isinstance(x, binary_type):
		return x
	else:
		return to_binary(str(x))  # str() can fail.  # do i need bytes() here with py3 (because ascii and stuff?) ?




def to_unicode(x):
	if isinstance(x, binary_type):
		return x.decode("utf-8")
	elif isinstance(x, text_type):
		return x
	else:
		return to_unicode(str(x))  # str() can fail.



# -*- coding: utf-8 -*-
__author__ = 'luckydonald'

from ..logger import logging  # pip install luckydonald-utils
logger = logging.getLogger(__name__)
from ..encoding import to_unicode

from django.http import JsonResponse, HttpResponse
from django.conf import settings
from django.utils.decorators import available_attrs
from functools import wraps


def json_response(content=None, status=None, statusText=None, exception=None):
	msg = {"status": 200, "statusText": "OK", "content": None}
	if isinstance(exception, Exception):
		logger.warn("Json Exception", exc_info=True)
		msg["status"] = 500
		msg["statusText"] = "INTERNAL SERVER ERROR"
		msg["content"] = "Generic Error"
		if settings.DEBUG:
			msg["exception"] = str(exception)
		#end if
	#end if
	if status:
		msg["status"] = int(status)
	if statusText:
		msg["statusText"] = statusText
	if content:
		msg["content"] = content
	#end if
	return JsonResponse(msg, status=msg["status"])



####
#
#  Render Exceptions
#
#

def render_all_exceptions(function):
	"""
	View decorator that renders Exception in django.
	If you want to catch and respond specific Exception types, see `render_specific_exception` decorator

	>>> @render_all_exceptions
	... def foobar()
	... 	pass # code goes here

	"""
	@wraps(function, assigned=available_attrs(function))
	def function_callable(request, *args, **kwargs):
		try:
			response = function(request, *args, **kwargs)
		except Exception as e:
			logger.debug("Exception response, because expected Exception occurred. {err_str}".format(err_str=str(e)))
			return HttpResponse(str(e), status=500)
		#end try
		return response
	return function_callable

def render_DoOutputException(function):
	"""
	View decorator that renders DoOutputException's message in django.
	If you want to catch and respond specific Exception types, see `render_specific_exception` decorator.
	If you want to catch and display all Exception types, see `render_all_exceptions` decorator

	>>> @render_DoOutputException
	... def foobar()
	... 	raise ValueError("lol.")

	"""
	@wraps(function, assigned=available_attrs(function))
	def function_callable(request, *args, **kwargs):
		try:
			response = function(request, *args, **kwargs)
		except DoOutputException as e:
			logger.debug("Exception response, because expected DoOutputException occurred. {err_str}".format(err_str=str(e)))
			return HttpResponse(str(e), status=500)
		#end try
		return response
	return function_callable


def render_specific_exception(exception_class, exception_render_func=None):
	"""
	 View decorator that allows to render specified Exception Types.
	You can specify a `exception_render_func` which will be called with the request and the exception as arguments.

	:param   exception: The Exception you want to be catched and rendered as response.
	:type    exception: class (inherits Exception)

	:keyword exception_render_func: an optional function to render the exception. It will be called with `func(request, e)`
	:type    exception_render_func: function (2 parameters [request, exception])

	:return: Django Response
	:rtype : HttpResponse




	Example:
	 @render_specific_exception(django.http.Http404)
		def view(request, ...):
			...

	For class-based views use:
	@method_decorator(render_specific_exception(django.http.Http404))
	 def get(self, request, ...)
		...

	Import it
	>>> from luckydonaldUtils.djangos.responses import render_specific_exception


	How to use
	>>> @render_specific_exception(AttributeError)
	... @render_specific_exception(ValueError, exception_render_func=foo_error_renderer)  # foo_error_renderer see below.
	... def django_foo(do_fail = 0):
	... 	if do_fail == 1:
	... 		raise AttributeError("ololol! Without custom renderer")
	... 	elif do_fail == 2:
	... 		raise ValueError("hua! With custom renderer.")
	... 	elif do_fail == 3:
	... 		raise AssertionError("Another test Error, not handled.")
	... 	return HttpResponse("normal output")

	Example exception_render_func function
	>>> def foo_error_renderer(request, e):
	... 	return HttpResponse("ERROR " + str(e) + " YEAH!") # but no status=500

	Try it:
	>>> django_foo(0)
	>>> django_foo(1)
	>>> django_foo(2)
	>>> django_foo(3)
	"""
	def create_function(function):
		@wraps(function, assigned=available_attrs(function))
		def function_callable(request, *args, **kwargs):
			try:
				response = function(request, *args, **kwargs)
			except exception_class as e:
				logger.debug("Exception response, because expected Exception occurred. {err_str}".format(err_str=str(e)))
				if exception_render_func:
					return exception_render_func(request, e)
				return HttpResponse(str(e), status=500)
			#end try
			return response
		return function_callable
	return create_function

class DoOutputException(Exception):
	pass


# -*- coding: utf-8 -*-

__author__ = 'luckydonald'

from ..dependencies import import_or_install
import_or_install("PIL", "Pillow")
from PIL import Image


def most_frequent_color(image, colors=10):
	image2 = image.convert("P", palette=Image.ADAPTIVE, colors=colors)
	image3 = image2.convert(image.mode)
	del image2

	w, h = image3.size
	pixels = image3.getcolors(w * h)

	most_frequent_pixels = [pixels[0]]

	for count, color in pixels:
		length = len(most_frequent_pixels)
		for i in range(0, length):
			if color == most_frequent_pixels[i][1]:
				break
			if count > most_frequent_pixels[i][0]:
				most_frequent_pixels.insert(i, (count, color))
				break
			elif count < most_frequent_pixels[length-1]:
				most_frequent_pixels.append((count, color))
				break
	del image3
	return most_frequent_pixels[:colors]